/**
 * imports
 */

import MenuItem from './menu_item.js';

/**
 * menu storage
 * @param string name
 * @param string url
 * @param bool active
 * @param bool nuke
 * @param array menu_items
 */
function Menu (name, url, active, nuke, menu_items) {
	
	this.name   = name;
	this.url    = url;
	this.active = active;
	this.nuke   = nuke;
	
	this.menu_items = menu_items;
};

/**
 * get an entry with this name
 * @param mixed item either a Menu | MenuItem
 * @return Menu for fluent interface
 */
Menu.prototype.add = function (item) {
	
	this.menu_items.push( item );
	
	return this;
};

/**
 * build the active list for persisting state
 * @return object 
 */
Menu.prototype.build_active_list = function () {

	const result = {};
	
	for (let k = 0; k < this.menu_items.length; k++) {
		
		const ooo = this.menu_items[k];
		
		console.log( ooo.name );
		
		if (ooo instanceof MenuItem) {
			// corner case - top level
			result[ooo.name] = (ooo.active ? '1' : '0');
			
		} else if (ooo instanceof Menu) {
			// sub menu
			result[ooo.name] = ooo.build_active_list();
			
		} else {
			console.error( 'unknown menu entry - build_active_list', ooo);
		}
	}
	
	return result;
};

/**
 * build the nav list for persisting state
 * @return object 
 */
Menu.prototype.build_nav_list = function (arg) {
	
	const result = {};

	for (let k = 0; k < this.menu_items.length; k++) {

		const ooo = this.menu_items[k];

		if (ooo instanceof MenuItem) {
			
			result[ooo.name] = ooo.url;
			
		} else if (ooo instanceof Menu) {
			
			result[ooo.name] = ooo.build_nav_list();
			
		} else {
			console.error( 'unknown menu entry - build_nav_list', ooo);
		}
	}

	return result;
};

/**
 * build the nuke list for persisting state
 * @return object 
 */
Menu.prototype.build_nuke_list = function (arg) {
	
	const result = {};
	
	for (let k = 0; k < this.menu_items.length; k++) {
		
		const ooo = this.menu_items[k];
		
		if (ooo instanceof MenuItem) {
			// corner case - top level
			result[ooo.name] = (ooo.nuke ? '1' : '0');
			
		} else if (ooo instanceof Menu) {
			// sub menu
			result[ooo.name] = ooo.build_nuke_list();
			
		} else {
			console.error( 'unknown menu entry - build_nuke_list', ooo);
		}
	}
	
	return result;
};

/**
 * comparison operator
 * @param mixed $arg
 * @return boolean
 */
Menu.prototype.equals = function (arg) {
	
	let result = false;
	
	if (arg instanceof Menu) {
		
		result = ((this.name == arg.name) && (this.url == arg.url));
	}
	
	return result;
};

/**
 * get an entry with this name
 * @param mixed item either a Menu | MenuItem
 * @return mixed Menu | MenuItem | bool
 */
Menu.prototype.find_by_item = function (item) {
	
	const detected = this.menu_items.findIndex(
		function (arg) {
			return (arg == item);
		}
	);
	
	let result = false;
	
	if (detected > -1) {
		result = this.menu_items[ detected ];
	}
	
	return result;
};

/**
 * get an entry with this name
 * @param string name
 * @return mixed Menu | MenuItem | bool
 */
Menu.prototype.find_by_name = function (name) {
	
	const detected = this.menu_items.findIndex(
		function (arg) {
			return ((arg.name == name) && (arg instanceof Menu));
		}
	);
	
	let result = false;
	
	if (detected > -1) {
		result = this.menu_items[ detected ];
	}
	
	return result;
};

/**
 * does the menu with this name exist?
 * @param string name
 * @return bool
 */
Menu.prototype.has = function (name) {
	
	const detected = this.has_menu(name) || this.has_menu_item(name);
	
	return detected;
};

/**
 * does the menu with this name exist?
 * @param string name
 * @return bool
 */
Menu.prototype.has_menu = function (name) {
	
	// ensure that top level "all" name is present
	const detected = this.menu_items.findIndex(
		function (arg) {
			return ((arg.name == name) && (arg instanceof Menu));
		}
	);
	
	const result = (detected > -1);
	
	return result;
};

/**
 * does the menu=-item with this name exist?
 * @param string name
 * @return bool
 */
Menu.prototype.has_menu_item = function (name) {
	
	const detected = this.menu_items.findIndex(
		function (arg) {
			return ((arg.name == name) && (arg instanceof MenuItem));
		}
	);
	
	const result = (detected > -1);
	
	return result;
};

/**
 * pretty print the contents
 * @param int offset
 * @return string
 */
Menu.prototype.pretty_print = function (offset = 0) {
	
	const spacer = "=> ".repeat(offset);
	
	let result =
		spacer + "Menu " +
		JSON.stringify(
			{
				name:   this.name,
				url:    this.url,
				active: this.active,
				nuke:   this.nuke
			}
		) +
		"\n";
	
	for (let k = 0; k < this.menu_items.length; k++) {
		
		const item = this.menu_items[k];
		
		result += item.pretty_print( offset + 1);
	}
	
	return result;
};

/**
 * set/clear the active flag for menu
 * @param bool flag
 * @return Menu for fluent interface
 */
Menu.prototype.set_active = function (flag) {
	
	this.active = flag;
	
	for (let k = 0; k < this.menu_items.length; k++) {
		
		const item = this.menu_items[k];
		
		item.set_active(flag);
	}
	
	return this;
};

/**
 * set/clear the deleted flag for menu
 * @param bool flag
 * @return Menu for fluent interface
 */
Menu.prototype.set_nuked = function (flag) {
	
	this.nuke = flag;
	
	//console.log( 'set_nuked menu', this.name, this.nuke );
	
	for (let k = 0; k < this.menu_items.length; k++) {
		
		const item = this.menu_items[k];
		
		item.set_nuked(flag);
	}
	
	return this;
};
 

/**
 * exports
 */
export default Menu;
