<?php
/* TOP_COMMENT_START
 * Copyright (C) 2022, Levion Partners dba PulseCMS - All Rights Reserved
 *
 * This file is part of Pulse Core. It may be used by individuals or organizations generating less than $400,000 USD per year in revenue, free-of-charge. Individuals or organizations generating over $400,000 in annual revenue who continue to use Pulse Core after 90 days for non-evaluation and non-development use must purchase a paid license. 
 *
 * Proprietary
 * You may modify this source code for internal use. Resale or redistribution is prohibited.
 *
 * You can get the latest version at: https://pulsecms.com/
 *
 * Dated June 2023
 *
TOP_COMMENT_END */

namespace pulsecore\image;

/**
 * get some information on the image filename
 * @param string $source The source image
 * @return stdClass
 */
function info (string $source) : \stdClass {
	
	\pulsecore\pre_condition( \strlen($source) > 0);
	
	$extended_info = array();
	
	$info = \getimagesize( $source, $extended_info );
	
	\pulsecore\invariant( \is_array($info) );
	
	$result = new \stdClass();
	$result->width  = $info[0];
	$result->height = $info[1];
	$result->type   = $info[2];
	$result->mime   = $info['mime'];
	
	return $result;
}

/**
 * test to see if the file is a supported image type
 * @param string $source The source image
 * @return bool
 */
function is_image (string $source) : bool {
	
	\pulsecore\pre_condition( \strlen($source) > 0);
	
	$extended_info = array();
	
	$info = \getimagesize( $source, $extended_info );
	
	$result = !(false === $info);
	
	return $result;
}

/**
 * load an image from filename
 * @param string $source The source image
 * @return resource The GD resource
 */
function load (string $source ) {
	
	\pulsecore\pre_condition( \strlen($source) > 0);
	
	$info = \pulsecore\image\info( $source );
	
	$result = false;
	
	switch ($info->type) {
		
		case \IMAGETYPE_GIF:
			$result = \imagecreatefromgif($source);
			break;
		
		case \IMAGETYPE_JPEG:
			$result = \imagecreatefromjpeg($source);
			break;
		
		case \IMAGETYPE_PNG:
			$result = \imagecreatefrompng($source);
			break;
	}
	
	\pulsecore\invariant( $result !== false );
	
	return $result;
}

/**
 * scale an image
 * @param string $source The source image
 * @param string $destination The destination filename
 * @param int $target_width The width of the result
 * @param int $target_height The height of the result
 * @return void
 */
function scale (string $source, string $destination, int $target_width, int $target_height) {
	
	\pulsecore\pre_condition( \strlen($source)      > 0);
	\pulsecore\pre_condition( \strlen($destination) > 0);
	
	\pulsecore\pre_condition( \intval($target_width)  > 0);
	\pulsecore\pre_condition( \intval($target_height) > 0);
	
	$img_src = \pulsecore\image\load($source);
	
	$img_dest = \imagescale(
		$img_src,
		$target_width, $target_height
	);
	
	\imagejpeg($img_dest, $destination, 100); # was \pulsecore\get_configs()->jpeg_quality
	
	#cleanup
	\imagedestroy( $img_dest );
	\imagedestroy( $img_src  );
}

/**
 * create thumbnail of an image
 * @param string $source The source image
 * @param string $destination The destination filename NB the destination name has the extension replaced by thumbnail.extension
 * @param int $target_width The width of the result
 * @param int $target_height The height of the result
 * @return void
 */
function thumbnail (string $source, string $destination, int $target_width, int $target_height) {
	
	\pulsecore\pre_condition( \strlen($source)      > 0);
	\pulsecore\pre_condition( \strlen($destination) > 0);
	
	\pulsecore\pre_condition( \intval($target_width)  > 0);
	\pulsecore\pre_condition( \intval($target_height) > 0);
	
	$thumbnail_image = thumbnail_path( $destination, $source );
	
	\pulsecore\image\scale( $source, $thumbnail_image, $target_width, $target_height );
}

/**
 * create thumbnail path for an image file string
 * @param string $destination The destination filename NB the destination name has the extension replaced by thumbnail.extension
 * @param string $owner_image_path The image the thumbnail was created from
 * @return string
 */
function thumbnail_path (string $destination, string $owner_image_path) : string {
	
	\pulsecore\pre_condition( \strlen($destination)      > 0);
	\pulsecore\pre_condition( \strlen($owner_image_path) > 0);
	
	$result = $destination;
	
	$file_info = \pathinfo( $result );
	$ext       = $file_info['extension'];
	
	\pulsecore\invariant(      isset($ext)     );
	\pulsecore\invariant( \is_string($ext)     );
	\pulsecore\invariant(    \strlen($ext) > 0 );
	
	# generate the hash to ensure unique thumbs
	$clean_owner_image_path = $owner_image_path;
	$clean_owner_image_path = \substr( $clean_owner_image_path, \stripos($clean_owner_image_path, 'content/') );
	$clean_owner_image_path = \str_replace( $ext, '', $clean_owner_image_path );
	$hash = \sha1($clean_owner_image_path);
	
	$result = \str_replace( ".{$ext}",  ".thumbnail.{$hash}.jpg",  $result );
	
	return $result;
}

