<?php
/* TOP_COMMENT_START
 * Copyright (C) 2022, Levion Partners dba PulseCMS - All Rights Reserved
 *
 * This file is part of Pulse Core. It may be used by individuals or organizations generating less than $400,000 USD per year in revenue, free-of-charge. Individuals or organizations generating over $400,000 in annual revenue who continue to use Pulse Core after 90 days for non-evaluation and non-development use must purchase a paid license. 
 *
 * Proprietary
 * You may modify this source code for internal use. Resale or redistribution is prohibited.
 *
 * You can get the latest version at: https://pulsecms.com/
 *
 * Dated June 2023
 *
TOP_COMMENT_END */

namespace pulsecore\installer;

/**
 * generate directory meta data
 */
abstract class BaseCollection extends Base implements \Iterator {
	
	/**
	 * the files and or directories
	 */
	protected $meta = false;
	
	/**
	 * constructor
	 * @param string $base_directory
	 */
	public function __construct (string $base_directory) {
		
		parent::__construct( $base_directory );
		
		$this->meta = [];
	}
	
	/**
	 * add an entry
	 * @param string $entry
	 * @return void
	 */
	public function add (string $entry) {
		
		\pulsecore\pre_condition( \strlen($entry) > 0);
		
		if (\is_file($entry)) {
			$m = new \pulsecore\installer\FileMetaData( $this->base_directory, $entry );
			
		} else if (\is_dir($entry)) {
			$m = new \pulsecore\installer\DirectoryMetaData( $this->base_directory, $entry );
		}
		
		$this->insert( $m );
	}
	
	/**
	 * add entries INSIDE a directory
	 * @param string $entry
	 * @return void
	 */
	public function add_dir (string $entry) {
		
		\pulsecore\pre_condition( \strlen($entry) > 0);
		
		$iter = new \FilesystemIterator( $entry, \FilesystemIterator::CURRENT_AS_PATHNAME | \FilesystemIterator::SKIP_DOTS );
		
		while ($iter->valid()) {
			
			$f = $iter->current();
			
			$this->add( $f );
			
			$iter->next();
		}
	}
	
	/**
	 * add an entry meta data - creates directories as needed
	 * @param string $filename NB the base dir MUST already be removed
	 * @param string $meta_data_type
	 * @param string $hash
	 * @return void
	 */
	public function add_meta (string $filename, string $meta_data_type, string $hash = '') {
		
		\pulsecore\pre_condition( \strlen($filename)       >  0);
		\pulsecore\pre_condition( \strlen($meta_data_type) >  0);
		\pulsecore\pre_condition( \strlen($hash)           >= 0);
		
		\pulsecore\pre_condition( \in_array( $meta_data_type, ['file', 'directory']) );
		
		# add to directory as needed
		$path = \ltrim( $filename, '/' );
		
		$path = \explode( '/', $path );
		
		$ptr = $this;
		
		$collect = '';
		
		while (sizeof($path) > 0) {
			
			$ppp = \array_shift($path);
			
			$collect = $collect . '/' . $ppp;
			$collect = \ltrim( $collect, '/' );
			
			if (\strlen($ppp) > 0) {
				
				if ($ptr->has($ppp)) {
					
					$ptr = $ptr->get($ppp);
					
				} else {
					
					# must be a directory
					if (\sizeof($path) > 0) {
						$zzzz =  new \pulsecore\installer\DirectoryMetaData( $this->base_directory, $collect );
						
						$ptr->insert( $zzzz );
						
						$ptr = $zzzz;
						
					}
				}
			}
		}
		
		# directory already added - so only add files
		if ($meta_data_type == 'file') {
			$m = new \pulsecore\installer\FileMetaData( $this->base_directory, $filename, $hash );
			
			$ptr->insert( $m );
			
		} else if ($meta_data_type == 'directory') {
			#$m = new \pulsecore\installer\DirectoryMetaData( $this->base_directory, $filename );
		}
	}
	
	/**
	 * compare against other meta-data collections
	 * @param \pulsecore\installer\BaseCollection $other
	 * @return array with deleted/common/new keys Deleted is anything not in
	 * $other, changed is for common files which are different, identical holds unchanged entries in both, and new is anything in $other
	 * thats not this object
	 */
	public function diff (\pulsecore\installer\BaseCollection $other) : array {
		
		$result = [
			'deleted'   => [],
			'changed'   => [],
			'identical' => [],
			'new'       => []
		];

		
		foreach ($this->meta as $key_a => $value_a) {
			
			# matching entries
			if ($other->has($key_a)) {
				
				$value_b = $other->get($key_a);
			
				# directories
				if ($value_a instanceof \pulsecore\installer\DirectoryMetaData) {
					
					$dc = $value_a->diff( $value_b );
					
					# splice in
					foreach (['deleted', 'changed', 'identical', 'new'] as $ddd) {
						foreach ($dc[$ddd] as $dc_key => $dc_value) {
							$result[$ddd][ $dc_key /*$key_a . '/' . $dc_value*/ ] = $dc_value;
						}
					}
					
				}
				
				# files
				if ($value_a instanceof \pulsecore\installer\FileMetaData) {
					
					$cmp = $value_a->compare( $value_b );
					
					if ($cmp) {
						$result['identical'][ $value_a->label() ] = $value_a;
					} else {
						$result['changed'][ $value_a->label() ] = $value_a;
					}
				}
					
			} else {
				
				# key_a not in $other
				$result['deleted'][ $value_a->label() ] = $value_a;
			}
		}
		
		# new files - things in other but not here
		foreach ($other->meta as $key_b => $value_b) {
			
			# something new
			if (!$this->has($key_b)) {
				
				$result['new'][ $value_b->label() ] = $value_b;
				
			}
		}
		
		return $result;
	}
	
	/**
	 * find an entry in the collection
	 * @param string $path
	 * @return \pulsecore\installer\Base
	 */
	public function find ($path) : \pulsecore\installer\Base {
		
		$cleaned = \ltrim( $path, '/' );
		
		$splitted = \explode( '/', $cleaned );
		
		$ptr = $this;
		
		$collect = '';
		
		foreach ($splitted as $value) {
			
			$collect = $collect . '/' . $value;
			$collect = \ltrim($collect, '/');
			
			\pulsecore\invariant( $ptr->has($collect) );
			
			$ptr = $ptr->get( $collect );
		}
		
		return $ptr;
	}
	
	/**
	 * get an entry - top level only
	 * @param string $name
	 * @return string
	 */
	public function get (string $name) : \pulsecore\installer\Base {
		
		\pulsecore\pre_condition( $this->has($name) );
		
		$result = $this->meta[ $name ];
		
		return $result;
	}
	
	/**
	 * does an entry exist ? Top level only
	 * @return string
	 */
	public function has ($key) : bool {
		
		$result = isset( $this->meta[ $key ] );
		
		return $result;
	}
	
	/**
	 * add an entry - as object
	 * @param \pulsecore\installer\Base $entry
	 * @return void
	 */
	public function insert (\pulsecore\installer\Base $entry) {
		
		$this->meta[ $entry->basename() ] = $entry;
	}
	
	/**
	 * Iterator method
	 * @return mixed
	 */
	public function current () {
		
		return \current( $this->meta );
	}
	
	/**
	 * Iterator method
	 */
	public function key () {
		return \key( $this->meta );
	}
	
	/**
	 * Iterator method
	 */
	public function next () {
		\next($this->meta);
	}
	
	/**
	 * Iterator method
	 */
	public function rewind () {
		\reset( $this->meta );
	}
	
	/**
	 * Iterator method
	 * @return bool
	 */
	public function valid () {
		
		$position = $this->key();
		
		$result = isset($this->meta[ $position ]);
		
		return $result;
	}
}
