<?php
/* TOP_COMMENT_START
 * Copyright (C) 2022, Levion Partners dba PulseCMS - All Rights Reserved
 *
 * This file is part of Pulse Core. It may be used by individuals or organizations generating less than $400,000 USD per year in revenue, free-of-charge. Individuals or organizations generating over $400,000 in annual revenue who continue to use Pulse Core after 90 days for non-evaluation and non-development use must purchase a paid license. 
 *
 * Proprietary
 * You may modify this source code for internal use. Resale or redistribution is prohibited.
 *
 * You can get the latest version at: https://pulsecms.com/
 *
 * Dated June 2023
 *
TOP_COMMENT_END */

namespace pulsecore\store\block;

/**
 * block list storage handler
 */
class Pile extends Base {
	
	/*
	 * the directory containing the block items
	 */
	protected $directory = '';
	
	/*
	 * the location of the directory (relative to the block directory)
	 */
	protected $location = false;
	
	/*
	 * the name of the block
	 */
	protected $name = '';
	
	/*
	 * construct
	 * \param $directory string the directory containing the block
	 * \param $params array eg filter tags
	 */
	function __construct( $directory, array $params = array() ) {
		
		\pulsecore\pre_condition(      isset($directory) );
		\pulsecore\pre_condition( \is_string($directory) );
		\pulsecore\pre_condition(    \strlen($directory) > 0);
		
		$this->directory = \trim($directory);
		
		$this->location = Pile::extract_relative_directory( $directory );
		
		$this->name = \basename( $this->directory );
		
		$this->params = $params;
		
		\pulsecore\invariant(      isset($this->name) );
		\pulsecore\invariant( \is_string($this->name) );
		\pulsecore\invariant(    \strlen($this->name) > 0);
	}
	
	/*
	 * flatten a list of piles from list_piles into a list
	 * \param $arg Pile object
	 * \return array
	 */
	public static function flatten( Pile $arg ) {
		
		$result = array();
		
		$result[] = $arg;
		
		foreach ($arg->sub_piles() as $foo) {
			
			$tmp = Pile::flatten( $foo );
			
			$result = \array_merge( $result, $tmp );
		}
		
		return $result;;
	}
	
	/*
	 * generate a new item clean basename for new items
	 * \return string
	 */
	public static function generate_clean_item_name() {
		
		$result = new \DateTime();
		$result = $result->format( 'YmdHis' );
		
		return $result;
	}
	
	/*
	 * get the location
	 * \return string
	 */
	public function get_location() {
		return $this->location;
	}
	
	/*
	 * list the block items in the roll
	 * \param $page_number integer 
	 * \param $page_size integer
	 * \return array of \pulsecore\store\block\Item objects
	 */
	public function items( $page_number, $page_size ) {
		
		\pulsecore\pre_condition(       isset($page_number) );
		\pulsecore\pre_condition( \is_numeric($page_number) );
		\pulsecore\pre_condition(     \intval($page_number) > 0);
		
		\pulsecore\pre_condition(       isset($page_size) );
		\pulsecore\pre_condition( \is_numeric($page_size) );
		\pulsecore\pre_condition(     \intval($page_size) >= 0);
		
		# process
		$result = array();
		
		if (\intval($page_size) > 0) {
			
			$items = \glob( $this->directory . '/*' );
			
			$pagination_start = ($page_number - 1) * $page_size;
			$pagination_end   =  $page_number      * $page_size; 
			
			$counter = 0;
			
			foreach ($items as $value) {
				
				if (($counter >= $pagination_start) and ($counter < $pagination_end)) {
				
					# item files
					if (\is_file($value)) {
						
						$item = $this->item_load( $value );
						
						$result[] = $item;
						
						$counter++;
					}
					
					# directory - skip
					# if (\is_dir($value)) {
					# }
				}
			}
		}
		
		return $result;
	}
	
	/*
	 * parse block item file
	 * \param $filename string
	 * \return \pulsecore\store\block\Item object
	 */
	public function item_load( $filename ) {
		
		\pulsecore\pre_condition(      isset($filename) );
		\pulsecore\pre_condition( \is_string($filename) );
		\pulsecore\pre_condition(    \strlen($filename) > 0);
		
		$filename = \trim( $filename );
		
		$result = new Item();
		$result->load( $filename );
		
		return $result;
	}
	
	/*
	 * save a block item to file
	 * @param string $filename
	 * @param \pulsecore\store\block\Item $item
	 * @return void
	 */
	public function item_save (string $filename, Item $item ) {
		
		\pulsecore\pre_condition(\strlen($filename) > 0);
		
		$item->save( $filename );
	}
	
	/*
	 * number of items in the block roll
	 * \return integer
	 */
	public function size() {
		
		# process
		$items = \glob( $this->directory . '/*.txt' );
		
		$result = \sizeof($items);
		
		return $result;
	}
	
	/*
	 * list sub diretory for blocks in the pile
	 * \param $directory string
	 * \return array of \pulsecore\store\block\Pile objects
	 */
	public function sub_piles() {
		
		$result = array();
		
		$items = \glob( $this->directory . '/*' );
		
		foreach ($items as $value) {
			
			# blog items - skip
			# if (\is_file($value)) {
			# }
			
			# directory
			if (\is_dir($value)) {
				$result[] = new Pile($value);
			}
		}
		
		return $result;
	}
}
